line_ending_or_eof

Function line_ending_or_eof 

fn line_ending_or_eof<'a>(input: &mut &'a str) -> ModalResult<&'a str>
Parses either a line ending or eof from an input.

Errors

Returns an error if input contains neither a line ending nor eof.