fn full_source<'i>(input: &Input<'i>) -> &'i strExpand description
Get a reference to the full original source from an Input.
Internally, Input stores the full initial input. By copying its reference and
resetting the pointer to the start, we get the the original string slice that is being parsed.
This is needed, as the offsets of each layer are relative to the start of the full string slice.