pub struct ParseStack<'i> {
pub(crate) at: usize,
pub external: Option<Arc<dyn Error + Send + Sync>>,
pub(crate) pending: Vec<StringContext>,
pub(crate) layers: Vec<Layer>,
pub(crate) source: &'i str,
}Expand description
A custom nested and span-aware parser error.
This error type is used across the ALPM project and provides well-formated and detailed parsing
errors. It allows addition of context per layer/type. For more information on the formatting see
the Display impl of ParseStack.
Fields
at: usizeByte offset of the deepest failure within the input.
external: Option<Arc<dyn Error + Send + Sync>>An external error, for the case that the final parser failure happened outside of winnow.
pending: Vec<StringContext>The StringContext instances that are not yet captured by a layer boundary.
layers: Vec<Layer>All closed nesting layers, innermost first.
source: &'i strThe full original source text. Required for rendering.
Implementations
Sourceimpl<'i> ParseStack<'i>
impl<'i> ParseStack<'i>
Sourcepub(crate) fn close_layer(self, name: String, start: usize) -> Self
pub(crate) fn close_layer(self, name: String, start: usize) -> Self
Moves all pending context calls into a new layer named name.
This is called by the LayerParser in case an error unwinds.
Sourcepub(crate) fn innermost(&self) -> LayerRef<'_>
pub(crate) fn innermost(&self) -> LayerRef<'_>
Returns the innermost layer.
Pending context belongs to the outermost “anonymous” layer that’s not yet named. If no named layer can be found, but there’s some pending context, we return the pending context as an anonymous layer as a fallback.
Sourcepub(crate) fn layer_stack(&self) -> Vec<LayerRef<'_>>
pub(crate) fn layer_stack(&self) -> Vec<LayerRef<'_>>
Returns the stack from outermost to innermost.
Pending context is positioned first as an anonymous outermost layer.
Sourcepub(crate) fn innermost_expected(&self) -> Vec<String>
pub(crate) fn innermost_expected(&self) -> Vec<String>
Returns all “expected” style contexts of the innermost layer that has any.
The layers are searched from the innermost outwards, with pending context being the outermost anonymous layer. If there’s no hit on the innermost layer, we just take the next best layer with such context to at least provide some information to the user.
Returns an empty list if no layer has any “expected” style context.
Sourcepub(crate) fn headline(&self) -> String
pub(crate) fn headline(&self) -> String
Returns the “headline” for the error.
Tries to get the best match available in the following order:
StringContext::Labelname of the innermost layer- Name of first non-anonymous string
- A simple fallback to the static string
"invalid".
Sourcefn anonymous_layer(&self) -> LayerRef<'_>
fn anonymous_layer(&self) -> LayerRef<'_>
Takes the current pending context and return it as an anonymous layer.